# File: CDF_CHEM.R
# Purpose: Population estimation for Region 10 and states for Chemistry
#          Data files include final design information and
#          only have one record for each site from the probability sample
# Programmer: Tony Olsen
# Date: October 11, 2005
# Modified: Dan McKenzie
# Date: January 27, 2006
# Data required: chem_wgt3.csv
#
# Libraries required: psurvey.analysis 2.9
# R scripts required:
#      cont.ash.r
#      ash1.wgt.r
#      cont.plot.r
#      plot.cont.r
#      plotcdf.r

# load psurvey.analysis
require('psurvey.analysis')
# source required R scripts
source('plot.cont.r')
source('plotcdf.r')
source('cont.plot.r')
source('cont.ash.r')
source('ash1.wgt.r')


#### Example Population estimates for Chemistry continuous variables
# Read in status-weight-data file 
chem <- read.csv('Original Data/chem_wgt3.csv')
head(chem)
dim(chem)
names(chem)

# add equal area coordinates for local variance estimation
tmp <- marinus(chem$POPLAT_DD, chem$POPLONG_DD)
chem$xmarinus <- tmp[,'x']
chem$ymarinus <- tmp[,'y']


# Set up data frames for cdf estimation
sites.chem <- data.frame(SiteID=chem$SITE_ID, 
                  Use=(chem$STATUS_1=='TS' & chem$SAMPCHEM=='Yes'))
subpop.chem <- data.frame(SiteID=chem$SITE_ID, 
                          Region10=rep('Region 10', nrow(chem)),
                          State=chem$STATE
                          )
dsgn.chem <- data.frame(siteID=chem$SITE_ID, 
                        stratum=chem$STRATUM,
                        wgt=chem$WGT_COND, 
                        xcoord=chem$xmarinus,
                        ycoord=chem$ymarinus )
      
data.cont.chem <- data.frame(siteID=chem$SITE_ID,
                 pH=chem$PHSTVL, 
                 Nitrate=chem$NO3
                 )


# Do cdf on sites 
chemcdf <- cont.analysis(sites=sites.chem,
                  subpop=subpop.chem,
                  design=dsgn.chem,
                  data.cont=data.cont.chem, 
                  pctval=c(5, 10, 25, 50, 75, 90, 95)
                  )

write.table(chemcdf$Pct,'chem_pct.csv',sep = ",",col.names=NA)

# Estimate empirical density function for chemistry variables
# define ind.type frame for use in both density and plot functions
ind.type.chem <- rep('Continuous',2)

# below is for all variables
ind.type.chem <- rep('Continuous',20)

# define ab matrix for use in density function
# note:  set lower limit=0 for any variables that you will 
# be plotting using a logrithmic scale
ab.chem <- matrix(c(3,14,
                    0,700),
                  nc=2,
                  byrow=TRUE)


# ash density estimates                
density.chem <- cont.ash(sites=sites.chem, 
               subpop=subpop.chem,
               design=dsgn.chem,
               data.cont=data.cont.chem,
               nbin=100,
               ab=ab.chem,
               ind.type=ind.type.chem, 
               vartype = "Local", 
               )
 
#define x-axis labels
xlabels.chem <- c('pH', 'Nitrate (ueq/L)')
names(xlabels.chem) <-c('pH','Nitrate')

               

# plot all subpopulations and indicators
cont.plot(pdffile='chemistry_plots.pdf',chemcdf, denlist=density.chem,
          ind.type=ind.type.chem, figlab='chem', 
          xlab=xlabels.chem, ylab.u='Stream Length (km)',
          logx=c(FALSE, TRUE),
          width=7, height=8)
       
