# File: chem_cdf_ compare.R
# Purpose: Population estimation and
#          Example of Comparing CDFs
# Programmer: Tony Olsen
# Date: October 17, 2005
# Modified: Dan McKenzie
# Date:  January 30, 2006
# Data required: chem_wgt3.csv
#
# Libraries required: psurvey.analysis 2.9
# R scripts required:
#      cont.ash.r
#      ash1.wgt.r
#      cont.plot.r
#      plot.cont.r
#      plotcdf.r

# load psurvey.analysis
require('psurvey.analysis')
# source required R scripts
source('plot.cont.r')
source('plotcdf.r')
source('cont.plot.r')
source('cont.ash.r')
source('ash1.wgt.r')


#### Example Population estimates for Chemistry continuous variables
# Read in status-weight-data file 
chem <- read.csv('Original Data/chem_wgt3.csv')
head(chem)
dim(chem)
names(chem)

# add equal area coordinates for local variance estimation
tmp <- marinus(chem$POPLAT_DD, chem$POPLONG_DD)
chem$xmarinus <- tmp[,'x']
chem$ymarinus <- tmp[,'y']


# Set up data frames for cdf estimation
sites.chem <- data.frame(SiteID=chem$SITE_ID, 
                  Use=(chem$STATUS_1=='TS' & chem$SAMPCHEM=='Yes'))
subpop.chem <- data.frame(SiteID=chem$SITE_ID, 
                          Region10=rep('Region 10', nrow(chem)),
                          ECO3=chem$ECO3
                          )
dsgn.chem <- data.frame(siteID=chem$SITE_ID, 
                        stratum=chem$STRATUM,
                        wgt=chem$WGT_COND, 
                        xcoord=chem$xmarinus,
                        ycoord=chem$ymarinus )
      
data.cont.chem <- data.frame(siteID=chem$SITE_ID,
                 pH=chem$PHSTVL, 
                 Nitrate=chem$NO3
                 )

# Do cdf on sites 
eco.chemcdf <- cont.analysis(sites=sites.chem,
                  subpop=subpop.chem,
                  design=dsgn.chem,
                  data.cont=data.cont.chem, 
                  pctval=c(5, 10, 25, 50, 75, 90, 95)
                  )

write.table(eco.chemcdf$Pct,'chem_comp_pct.csv',sep = ",",col.names=NA)

# Estimate empirical density function for chemistry variables
# define ind.type frame for use in both density and plot functions
ind.type.chem <- rep('Continuous',2)

# below is for all variables
ind.type.chem <- rep('Continuous',20)

# define ab matrix for use in density function
# note:  set lower limit=0 for any variables that you will 
# be plotting using a logrithmic scale
ab.chem <- matrix(c(3,14,
                    0,700),
                  nc=2,
                  byrow=TRUE)

# ash density estimates                
density.chem <- cont.ash(sites=sites.chem, 
               subpop=subpop.chem,
               design=dsgn.chem,
               data.cont=data.cont.chem,
               nbin=100,
               ab=ab.chem,
               ind.type=ind.type.chem, 
               vartype = "Local", 
               )
 
#define x-axis labels
xlabels.chem <- c('pH', 'Nitrate (ueq/L)')
names(xlabels.chem) <-c('pH','Nitrate')

# plot all subpopulations and indicators
cont.plot(pdffile='chemistry_other_plots.pdf',eco.chemcdf, denlist=density.chem,
          ind.type=ind.type.chem, figlab='chem', 
          xlab=xlabels.chem, ylab.u='Stream Length (km)',
          logx=c(FALSE, TRUE),
          width=7, height=8)
       
# plot both Eco3 regions on same plot

# test for difference by eco3 region
# pH CDF test
boxplot(split(chem$PHSTVL, chem$ECO3) )

# If desired save the box plot as a pdf file
# Before proceeding
# Choose save as from File pulldown memu

table(cut(chem$PHSTVL, 
     breaks=quantile(chem$PHSTVL, probs=seq(0,1, 0.2)) , 
             include.lowest=TRUE), 
      chem$ECO3 )
tst <- chem$ECO3 == 'MT'
eco.mt <- list(z=chem$PHSTVL[tst], wgt=chem$WGT_COND[tst], 
                 x=chem$xmarinus[tst], y=chem$ymarinus[tst] )
tst <- chem$ECO3 == 'XE'
eco.xe <- list(z=chem$PHSTVL[tst], wgt=chem$WGT_COND[tst], 
                 x=chem$xmarinus[tst], y=chem$ymarinus[tst] )

cdf.test(eco.mt, eco.xe, 
          bounds=quantile(chem$PHSTVL, probs=seq(0,1, 0.2))[-1]
         )

# NO3 CDF test
boxplot(split(chem$NO3, chem$ECO3), log='y' )

# If desired save the box plot as a pdf file
# Before proceeding
# Choose save as from File pulldown memu

table(cut(chem$NO3, 
     breaks=quantile(chem$NO3, probs=seq(0,1, 0.2))[-1] , 
             include.lowest=TRUE), 
      chem$ECO3 )

tst <- chem$ECO3 == 'MT'
eco.mt <- list(z=chem$NO3[tst], wgt=chem$WGT_COND[tst], 
                 x=chem$xmarinus[tst], y=chem$ymarinus[tst] )
tst <- chem$ECO3 == 'XE'
eco.xe <- list(z=chem$NO3[tst], wgt=chem$WGT_COND[tst], 
                 x=chem$xmarinus[tst], y=chem$ymarinus[tst] )

cdf.test(eco.mt, eco.xe, 
         bounds=quantile(chem$NO3, probs=seq(0,1, 0.2))[-c(1:2)]
        )

# Plot MT and XE CDFs on same graph
source('cdfplottwo.r')

tst <- eco.chemcdf$CDF$Subpopulation == 'MT' & 
       eco.chemcdf$CDF$Indicator == 'pH' 
ph.mt.cdf <- eco.chemcdf$CDF[tst, c('Value', 'Estimate.P', 'LCB95Pct.P', 'UCB95Pct.P')]
tst <- eco.chemcdf$CDF$Subpopulation == 'XE' & 
       eco.chemcdf$CDF$Indicator == 'pH' 
ph.xe.cdf <- eco.chemcdf$CDF[tst, c('Value', 'Estimate.P', 'LCB95Pct.P', 'UCB95Pct.P')]

cdfplottwo(ph.mt.cdf, ph.xe.cdf,
    xlab='pH', ylab='Percent Stream Length',
    main='MT (1st) and XE (2nd) ECO3 Comparison' )

# If desired save the cdf plot as a pdf file
# Before proceeding
# Choose save as from File pulldown memu

tst <- eco.chemcdf$CDF$Subpopulation == 'MT' & 
       eco.chemcdf$CDF$Indicator == 'Nitrate' 
no3.mt.cdf <- eco.chemcdf$CDF[tst, c('Value', 'Estimate.P', 'LCB95Pct.P', 'UCB95Pct.P')]
tst <- eco.chemcdf$CDF$Subpopulation == 'XE' & 
       eco.chemcdf$CDF$Indicator == 'Nitrate' 
no3.xe.cdf <- eco.chemcdf$CDF[tst, c('Value', 'Estimate.P', 'LCB95Pct.P', 'UCB95Pct.P')]

cdfplottwo(no3.mt.cdf, no3.xe.cdf, log='x',
    xlab='Nitrate (ueq/L)', ylab='Percent Stream Length',
    main='MT (1st) and XE (2nd) ECO3 Comparison' )

# If desired save the plot as a pdf file
# Before proceeding
# Choose save as from File pulldown memu


warnings ()
