# File: cont.ash.r
# Purpose: Create ASH density estimates for indicators and subpopulations
# Programmer: Tony Olsen
# Date: April 3, 2005
#  Revised: April 11, 2005

cont.ash <- function(sites, subpop, design, data.cont,
                          mhist=NULL, nbin=50, ab=NULL, ind.type='Continuous', 
                          sigma = NULL, var.sigma = NULL, N.cluster = NULL, popsize = NULL, 
                          stage1size = NULL, support = NULL, swgt = NULL, 
                          swgt1 = NULL, unitsize = NULL,  vartype = "Local", 
                          conf = 95, pctval = c(5, 25, 50, 75, 95)) {

# Arguments:
#   sites      data frame consisting of two variables: the first variable is
#              site IDs, and the second variable is a logical vector indicating 
#              which sites to use in the analysis.
#
#	  subpop     data frame describing sets of populations and subpopulations 
#              for which estimates will be calculated. The first variable is 
#              site IDs. Each subsequent variable identifies a Type of population, 
#              where the variable name is used to identify Type. A Type variable 
#              identifies each site with one of the subpopulations of that Type.
#
#   design     design a data frame consisting of design variables. 
#              Variables should be named as follows:
#                 siteID = site IDs
#                 wgt = final adjusted weights, which are either the weights 
#                       for a single-stage sample or the stage two weights 
#                       for a two-stage sample
#                 xcoord = x-coordinates for location, which are 
#                          either the x-coordinates for a single-stage sample or 
#                          the stage two x-coordinates for a two-stage sample
#                 ycoord = y-coordinates for location, which are 
#                          either the y-coordinates for a single-stage sample 
#                          or the stage two y-coordinates for a two-stage sample
#                 stratum = the stratum codes
#                 cluster = the stage one sampling unit(primary sampling unit or cluster)
#                           codes
#                    wgt1 = final adjusted stage one weights
#                 xcoord1 = the stage one x-coordinates for location
#                 ycoord1 = the stage one y-coordinates for location 
#
#   data.cont  data frame of continuous response variables. The first variable is
#              site IDs. Subsequent variables are response variables. 
#              Missing data (NA) is allowed.
#
#       mhist  Number of average shifted histograms desired
#
#        nbin  Number of bins desired ("delta" bins)
#
#          ab  Matrix (2, ncol(data.cont)-1) containing ranges for density estimates.
#              NA indicates no limit
#
#    ind.type  Type of indicator: "Continuous" or "Integer" for each indicator.
#              Default is "Continuous"
#
# Results:
#              A list of lists. Primary list is by Type of subpopulation. 
#              Secondary list is subpopulations within Type.
#              Tertiary list is by Indicator from columns of data.cont
#              Final list is bin centers (x) and bin heights (y) from ASH
#              Example list: den[[subtype]][[subpop]][[indicator]]
#
# Requires psurvey.analysis library
require(psurvey.analysis)

# do same input check as cont.analysis function
# most of this is not currently used as ASH only computed for unequal probability data.
# For example, any stratification is ignored at this time.
        if (is.null(sites)) 
            stop("\nThe sites data frame must be provided.")
        if (!is.data.frame(sites)) 
            stop("\nThe sites argument must be a data frame.")
        if (is.null(subpop)) 
            stop("\nThe subpop data frame must be provided.")
        if (!is.data.frame(subpop)) 
            stop("\nThe subpop argument must be a data frame.")
        if (is.null(design)) 
            stop("\nThe design data frame must be provided.")
        if (!is.data.frame(design)) 
            stop("\nThe design argument must be a data frame.")
        if (is.null(data.cont)) 
            stop("\nThe data.cont data frame must be provided.")
        if (!is.data.frame(data.cont)) 
            stop("\nThe data.cont argument must be a data frame.")
        design.names <- names(design)
        temp <- match(design.names, c("siteID", "wgt", "xcoord", 
            "ycoord", "stratum", "cluster", "wgt1", "xcoord1", 
            "ycoord1"), nomatch = 0)
        if (any(temp == 0)) 
            stop("\nThe names used in the design data frame do not match the required names.")
        temp <- dframe.check(sites, design, subpop, NULL, data.cont, design.names)
        sites <- temp$sites
        subpop <- temp$subpop
        design <- temp$design
        data.cont <- temp$data.cont
#        subpop <- subpop[sites[, 2], ]
#        design <- design[sites[, 2], ]
#        data.cont <- data.cont[sites[, 2], ]
        if (!is.null(support)) 
            support <- support[sites[, 2]]
        if (!is.null(swgt)) 
            swgt <- swgt[sites[, 2]]
        if (!is.null(swgt1)) 
            swgt1 <- swgt1[sites[, 2]]
#        sites <- sites[sites[, 2], ]
        siteID <- design$siteID
        wgt <- design$wgt
        xcoord <- design$xcoord
        ycoord <- design$ycoord
        stratum <- design$stratum
        cluster <- design$cluster
        wgt1 <- design$wgt1
        xcoord1 <- design$xcoord1
        ycoord1 <- design$ycoord1
        stratum.ind <- length(unique(stratum)) > 1
        if (stratum.ind) {
            stratum <- factor(stratum)
            stratum.levels <- levels(stratum)
            nstrata <- length(stratum.levels)
        }
        else {
            stratum.levels <- NULL
            nstrata <- NULL
        }
        cluster.ind <- length(unique(cluster)) > 1
        if (cluster.ind) {
            if (stratum.ind) {
                cluster.in <- cluster
                cluster <- tapply(cluster, stratum, factor)
                cluster.levels <- sapply(cluster, levels, simplify = FALSE)
                ncluster <- sapply(cluster.levels, length)
            }
            else {
                cluster <- factor(cluster)
                cluster.levels <- levels(cluster)
                ncluster <- length(cluster.levels)
            }
        }
        pcfactor.ind <- !is.null(support)
        swgt.ind <- !is.null(swgt)
 
        nresp <- dim(design)[1]
        temp <- input.check(nresp, wgt, sigma, var.sigma, xcoord, 
            ycoord, stratum.ind, stratum, stratum.levels, nstrata, 
            cluster.ind, cluster, cluster.levels, ncluster, N.cluster, 
            wgt1, xcoord1, ycoord1, popsize, stage1size, pcfactor.ind, 
            support, swgt.ind, swgt, swgt1, unitsize, vartype, 
            conf, pctval = pctval, subpop = subpop)
        N.cluster <- temp$N.cluster
        popsize <- temp$popsize
        stage1size <- temp$stage1size
        unitsize <- temp$unitsize
        if (stratum.ind && cluster.ind) 
            cluster <- cluster.in
        if (is.null(xcoord)) 
            xcoord <- rep(NA, nresp)
        if (is.null(ycoord)) 
            ycoord <- rep(NA, nresp)
        if (is.null(stratum)) 
            stratum <- rep(NA, nresp)
        if (is.null(cluster)) 
            cluster <- rep(NA, nresp)
        if (is.null(wgt1)) 
            wgt1 <- rep(NA, nresp)
        if (is.null(xcoord1)) 
            xcoord1 <- rep(NA, nresp)
        if (is.null(ycoord1)) 
            ycoord1 <- rep(NA, nresp)
        design <- data.frame(siteID = siteID, wgt = wgt, xcoord = xcoord, 
            ycoord = ycoord, stratum = stratum, cluster = cluster, 
            wgt1 = wgt1, xcoord1 = xcoord1, ycoord1 = ycoord1)

# compute ASH estimates
    nvar <- dim(data.cont)[2]
    varname <- names(data.cont)
    ntypes <- dim(subpop)[2]
    typenames <- names(subpop)
    # expand ind.type to nvar-1 if only single value given
    if(length(ind.type)==1) ind.type=rep(ind.type, nvar-1)
    rslt <- list()

    for (ivar in 2:nvar) {
        for (itype in 2:ntypes) {
            subpopnames <- levels(factor(subpop[, itype]))
            for (isubpop in 1:length(subpopnames)) {
                subpop.ind <- subpop[, itype] == subpopnames[isubpop]
                if (all(is.na(data.cont[subpop.ind, ivar]))) {
                  warning(paste("Subpopulation", subpopnames[isubpop], 
                    "of population type", typenames[itype], "for indicator", 
                    varname[ivar], "\ncontains no data.\n"))
                  next
                }
                if (sum(!is.na(data.cont[subpop.ind, ivar])) == 1) {
                  warning(paste("Subpopulation", subpopnames[isubpop], 
                    "of population type", typenames[itype], "for indicator", 
                    varname[ivar], "\ncontains a single value.  No analysis was performed.\n"))
                  next
                }
                if (stratum.ind) {
                  temp.N.cluster <- N.cluster[!is.na(match(names(N.cluster), 
                    unique(design[subpop.ind, 5])))]
                  temp.popsize <- popsize[[itype - 1]][[isubpop]][!is.na(match(names(popsize[[itype - 
                    1]][[isubpop]]), unique(design[subpop.ind, 
                    5])))]
                  temp.stage1size <- stage1size[!is.na(match(names(stage1size), 
                    unique(design[subpop.ind, 5])))]
                  temp.unitsize <- unitsize[!is.na(match(names(unitsize), 
                    unique(design[subpop.ind, 5])))]
                }
                else {
                  temp.N.cluster <- N.cluster
                  temp.popsize <- popsize[[itype - 1]][[isubpop]]
                  temp.stage1size <- stage1size
                  temp.unitsize <- unitsize
                }
                m.up <- function(n, nbin) {nbin/(30 * n)^(1/5) }
                if(is.null(mhist)) 
                   mhist <- floor(0.67 * m.up(length(data.cont[subpop.ind,ivar]), nbin))
             
                temp.ash <- ash1.wgt(x = data.cont[subpop.ind,ivar], 
                									wgt = design[subpop.ind, 2],
                									m=mhist, nbin=nbin, ab=ab[ivar-1,], 
                									support=ind.type[ivar-1]) 

#  Code from cont.analysis that may be useful when do ASH for more complex designs.                  
#                  x = design[subpop.ind,3], y = design[subpop.ind, 4], 
#                  stratum = design[subpop.ind,5], 
#                  cluster = design[subpop.ind, 6], N.cluster = temp.N.cluster, 
#                  wgt1 = design[subpop.ind, 7], x1 = design[subpop.ind,8], 
#                  y1 = design[subpop.ind, 9], popsize = temp.popsize, 
#                  stage1size = temp.stage1size, support = support[subpop.ind], 
#                  swgt = swgt[subpop.ind], swgt1 = swgt1[subpop.ind], 
#                  unitsize = temp.unitsize, vartype = vartype, 
#                  conf = conf)
              
           # add data to list of list of lists
               rslt[[typenames[itype]]][[subpopnames[isubpop]]][[varname[ivar]]] <- temp.ash

          } # end indicator loop

       } # end subpopulations within type loop
     } # end subpopulation Type loop
  return(rslt)
} # end of function

