# File: cont.plot.r
# Purpose: Create summary plots based on cont.analysis and cont.ash output files
# Programmer: Tony Olsen
# Date: April 11, 2005
#  Revised July 28, 2005

cont.plot <- function(pdffile='cdf2x2.pdf',cdflist, denlist, ind.type='Continuous',
                 xlab=NULL,ylab='Percent', ylab.u=NULL, figlab=NULL, logx='',
                 legloc='BR', confcut=5, conflev=95,  width=NULL, height=NULL, ...) {

# Input:
#	pdffile    file name for pdf file output
#	cdflist    CDF output data.frame from cont.analysis function
#	denlist    Density estimates to match CDF from cont.analysis.ash function
# ind.type   vector of length # of indicators with elements equal to "Continuous"
#            or "Integer".  Controls type of CDF plot and density plot
#	   xlab    label for x axis. vector with row.names to match indicator
#	   ylab    label for y axis
#   ylab.u   label for right-side y-axis
#   figlab   label for figure
#     logx   vector indicating whether x axis should be log scale (logx=TRUE) for 
#            each indicator
#   legloc   location for legend: BR=BottomRight, BL, TR, TL
#  confcut   Option to limit plotting of confidence limits at cdf extremes
#  conflev   Confidence level used for confidence limits.
#    width   width for pdf page in inches
#   height   height for pdf page in inches
 
### Do cdf plots by Type, then subpopulation within Type,
#    then Indicator within Subpopulation
k <- 0
pdf(file=pdffile, width=width, height=height)
typenames <- unique(cdflist$Pct$Type)
indnames <- unique(cdflist$Pct$Indicator)
# set up xlab if supplied
if( is.null(xlab) ) {
 xlab <- as.character(indnames)
 row.names(xlab) <- as.character(indnames)
}

if(length(ind.type) == 1) ind.type <- rep(ind.type, length(indnames))
if(is.null(logx) ) xlog <- rep('', length(indnames))
else {
  if(length(logx) == 1) {
    tmp <- ifelse(logx == TRUE, 'x', '')
    xlog <- rep(tmp, length(indnames) )
  }
  else {
  xlog <- rep('', length(indnames) )
  xlog[logx == TRUE] <- 'x'
  }
}
for(kin in 1:length(indnames)) {
  for(itype in 1:length(typenames)) {
     tsttype <- cdflist$Pct$Type==typenames[itype]
     tstind <- cdflist$Pct$Indicator==indnames[kin]
     subnames <- unique(cdflist$Pct$Subpopulation[tsttype & tstind])
     for(jsub in 1:length(subnames)) {
        tstsub <- tsttype & cdflist$Pct$Subpopulation==subnames[jsub]
	      tstind <- tstsub & cdflist$Pct$Indicator==indnames[kin]
	      k <- k + 1
	      if(is.null(figlab)) fignum <- NULL
	      else fignum <- paste(figlab,'-',k, sep='')
        plot.cont(subtype=as.character(typenames[itype]), 
                  subpop=as.character(subnames[jsub]), 
                  indicator=as.character(indnames[kin]), 
                  cdflist=cdflist, ind.den=denlist, ind.type[kin], logx=xlog[kin],
                  xlab=xlab[ indnames[kin] ], ylab=ylab, ylab.u=ylab.u,
                  figlab=fignum, confcut=confcut, conflev=conflev)

      }
   }
}

graphics.off()
# end of function
}

