#	File: plot.cont.r
# Purpose: One page summary for a single subpopulation and indicator
#          Based on cont.analysis and cont.ash output files
#	Programmer: Tony Olsen
#	Date: July 28, 2005
#
# Input:
# subtype    Type of subpopulation to be plotted
# subpop     name of subpopulation to be plotted (character)
# indicator  name of indicator to be plotted (character)
#	cdflist    list from cont.analysis function
# ind.den    ash1 function list for estimated density
# ind.type   Equal to "Continuous" or "Ordinal" 
# legloc     location of the legend in the cdf plot: BR - bottom:right,
#            BL - bottom:left, TR - top:right, TL - top:left
# ylab.u     Label for y-axis on right
# ...        graphical parameters 
#
# Output:
#	  Plot for a subpopulation and an indicator that consists of three parts:
#   (1) plot of cdf with confidence bounds
#   (2) summary statistics for percentiles, mean and std dev of distribution
#   (3) plot of estimated density based on a ash1 function

plot.cont <- function(subtype, subpop, indicator, cdflist, ind.den,
             ind.type='Continuous', legloc='BR', logx=NULL,
             xlab=NULL, ylab=NULL, ylab.u=NULL, figlab=NULL,
             confcut=5, conflev=95, ...) {

# Extract data from cont.analysis function list
tst <- cdflist$CDF$Type == subtype & cdflist$CDF$Subpopulation == subpop &
			 cdflist$CDF$Indicator == indicator
cdfest <- cdflist$CDF[tst,]
# create summary statistic data frame
tst <- cdflist$Pct$Type == subtype & cdflist$Pct$Subpopulation == subpop &
			 cdflist$Pct$Indicator == indicator
obj <- cdflist$Pct[tst,c('Statistic','Estimate', 'LCB95Pct', 'UCB95Pct')]
# drop variance estimate and rename std dev
obj <- obj[-(nrow(obj)-1),]
obj[nrow(obj),1] <- 'Std Dev'

# set up layout for plot
def.par <- par(no.readonly = TRUE) # save default, for resetting...
# nf <- layout(matrix(c(1,1,2,3,2,4), 3,2, byrow = TRUE), widths=c(5,3), heights=c(1,5,5))
nf <- layout(matrix(c(1,1,2,2,3,4), 3,2, byrow = TRUE), widths=c(4,4), heights=c(1,6,4))

# plot title
par(mgp=c(0,0,0), mar=c(0,0,0,1)+ 0.1 )
plot.new()
plot.window(c(0,1), c(0,1))
if(is.null(figlab) ) text(.5,.5,adj=0.5, cex=1.5, font=2,
     paste('Indicator:', indicator, '    Subpopulation:', subpop, sep=' ') )
#     paste('Indicator:', indicator, sep=' ') )
if(!is.null(figlab) ) text(0.0, 0.5, adj=0, cex=1.5, font=2,
    paste('Figure',figlab, 'Indicator:', indicator, '    Subpopulation:', subpop, sep=' ') )
#     paste('Figure',figlab, 'Indicator:', indicator, sep=' ') )

# plot CDF estimate
par(mgp=c(1.7,0.6,0), mar=c(3,3,2,4)+0.1 )
plot.cdf(cdfest, type.cdf=ind.type, units.cdf='percent',
             legloc='BR', confcut=confcut, conflev=conflev, 
             xlab=xlab, ylab=ylab, ylab.r=ylab.u, log=logx,
             figlab='Empirical Cumulative Distribution Estimate',...)

## plot summary statistics
par(mgp=c(0,0,0), mar=c(1,0,0,1)+ 0.1 )
plot.new()
xmax <- 15 + 20*(ncol(obj)-1 )
ymin <- 90 - 5*(nrow(obj) - 1)
plot.window(c(0,xmax), c(ymin,100))
text(50,100, adj=.5, 'Summary Statistics', font=2, cex=par('cex.main'))
clab <- c('Statistic', 'Est', 'LCB', 'UCB')
for(j in 1:ncol(obj) ) 
  text(15+20*(j-1), 95, adj=1, clab[j])
est <- round(obj[,-1], 2) 
for(i in 1:nrow(est) ) {
  text(15, 90-5*(i-1), adj=1, as.character(obj[i,1]))
  for(j in 1:ncol(est)) {
     text(15+20*j, 90-5*(i-1), adj=1,input.format(est[i,j],2,' ' ) )
  }
}

# plot density estimate
par(mgp=c(1.7,0.6,0), mar=c(4,3,3,2)+ 0.1 )
plot.new()
if(!is.null(ind.den) ) {
  rx <- range(ind.den[[subtype]][[subpop]][[indicator]]$x)
  ry <- range(ind.den[[subtype]][[subpop]][[indicator]]$y)
  plot.window(rx,ry , log=logx)
  axis(side=1 )
  box()
  title(main='Empirical Density Estimate', xlab=xlab )
  if(ind.type == 'Continuous') lines(ind.den[[subtype]][[subpop]][[indicator]])
  if(ind.type == 'Ordinal') {
    x <- ind.den[[subtype]][[subpop]][[indicator]]$x
    y <- ind.den[[subtype]][[subpop]][[indicator]]$y
    x <- matrix(t(matrix(c(x,x,rep(NA,length(x))),nc=3)), nc=1)
    y <- matrix(t(matrix(c(rep(0,length(y)),y,rep(NA,length(y))),nc=3) ), nc=1)
    lines(x, y)
  }
}
else plot.window(c(0,1), c(0,1))

# set par back
par(def.par)
}
