# File: plotcdf.r
# Purpose: Plot cdf and associated confidence limits
# Programmer: Tony Olsen
# Date: July 12, 2005

plot.cdf <- function(cdfest, type.cdf='continuous', units.cdf='percent',
             legloc='BR', logx=NULL, confcut=5, conflev=95, 
             xlab=NULL, ylab=NULL, ylab.r=NULL, 
             figlab=NULL,...) {

# Arguments
#     cdfest  Dataframe containing CDF estimate and confidence limits both in terms
#             of percent and actual units.  Structure is assumed to be the same
#             as cont.analysis function output in psurvey.analysis library. Must
#             contain only a single cdf estimate.
#
#   type.cdf  Is the variable "Continuous" or "Ordinal".  Changes form of cdf plot
#
#  units.cdf  If units.cdf="percent", then CDF plotted is Percent of population.
#             Otherwise CDF plotted is in terms of units of population.
#
#     legloc  location of the legend in the cdf plot: BR - bottom:right,
#             BL - bottom:left, TR - top:right, TL - top:left
#
#       logx  Option for plotting x axis on log10 scale (logx='x')
#
#    confcut  Option to limit plotting of confidence limits at cdf extremes.
#
#    conflev  Confidence level used for confidence limits.
#
#       xlab  Label for x-axis on bottom of plot
#
#       ylab  Label for y-axis on left side of plot
#
#     ylab.r  Label for y-axis on right side of plot. If ylab.r=NULL, then no
#             right side axis label.  If ylab.r="Same", then right side axis
#             label same as left side.  Otherwise right side label is text string given.
#             Note result depends on value of units.cdf.
#
#     figlab  Label for title of CDF plot
#             

# plot CDF estimate
op <- par(mgp=c(1.7,0.6,0), mar=c(3,3,2,4)+0.1 )

# plot cdf as percent or units
# column 1: xvalue, 2: cdf estimate for left axis, 
# 3: lower conf lim, 4: upper conf lim, 5: cdf estimate for right axis.
if(units.cdf=='percent') {
  ylim <- c(0,100)
  cdfdata <- cdfest[,c(4,6,8,9,10)]
}
else {
  cdfdata <- cdfest[,c(4,10,12,13,6)]
  ylim <- pretty(c(min(cdfdata[,c(2,3,4)]), max(cdfdata[,c(2,3,4)])) )[c(1,6)]
}

# restrict confidence limits between confcut and 100-confcut percent
pctval <- c(confcut, 100-confcut)
tvalue <- cdfest[,6]>=pctval[1] & cdfest[,6]<=pctval[2]
yhi <- interp.cdf(pctval, cdfest[,6], cdfdata[,4])
ylow <- interp.cdf(pctval, cdfest[,6], cdfdata[,3])
x <-  interp.cdf(pctval, cdfest[,6], cdfdata[,1])

# options for plotting count (ordinal) and continuous indicators
if(type.cdf == 'Continuous') {
  plot(cdfdata[,1],cdfdata[,2], type='l',ylim=ylim, 
       xlab=xlab, ylab=ylab, log=logx,...)
  # plot confidence limits
  value <- c(x[1], cdfdata[,1][tvalue], x[2])
  upper <- c(yhi[1], cdfdata[,4][tvalue], yhi[2])
  lower <- c(ylow[1], cdfdata[,3][tvalue], ylow[2])
  lines(value,lower,lty=3)
  lines(value,upper,lty=3)
  }
# plot count CDF
if(type.cdf == 'Ordinal') {
  # create horizontal line segments
  x <- rep(cdfdata[,1], each=2)[-1]
  y <- rep(cdfdata[,2], each=2)
  tmp <- cbind(matrix(c(x,x[length(x)]),nc=2,byrow=TRUE),rep(NA,nrow(cdfdata)))
  x <- as.vector(t(tmp))
  tmp <- cbind(matrix(y,nc=2,byrow=TRUE),rep(NA,nrow(cdfdata)))
  y <- as.vector(t(tmp))
  plot(x, y, type='l', ylim=ylim, xlab=xlab, ylab=ylab,...)
  # plot confidence limits
  len <- length(cdfdata[,1][tvalue])
  if(len > 1) {
    value <- rep(cdfdata[,1][tvalue], each=2)[-1]
    tmp <- cbind(matrix(c(value,value[length(value)]),nc=2,byrow=TRUE),rep(NA,len))
    value <- as.vector(t(tmp))
    len <- length(cdfdata[,4][tvalue])
    if(len > 1) {
      upper <- rep(cdfdata[,4][tvalue], each=2)
      tmp <- cbind(matrix(upper,nc=2,byrow=TRUE),rep(NA,len))
      upper <- as.vector(t(tmp))
      lower <- rep(cdfdata[,3][tvalue], each=2)
      tmp <- cbind(matrix(lower,nc=2,byrow=TRUE),rep(NA,len))
      lower <- as.vector(t(tmp))
      lines(value,lower,lty=3)
      lines(value,upper,lty=3)
    }
  }
}
# do title 
title(figlab, line=1)

# set up legend location
rx <- range(par('usr')[1:2], cdfdata[,1])
ry <- range(par('usr')[3:4], cdfdata[,2])
if(legloc=='BR') {
	xjust <- 1; yjust <- 0; 
	legx <- rx[2]; legy <- ry[1]
}
if(legloc=='BL') {
	xjust <- 0; yjust <- 0; 
	legx <- rx[1]; legy <- ry[1]
}
if(legloc=='TR') {
	xjust <- 1; yjust <- 1; 
	legx <- rx[2]; legy <- ry[2]
}
if(legloc=='TL') {
	xjust <- 0; yjust <- 1; 
	legx <- rx[1]; legy <- ry[2]
}
legend(x=legx,y=legy,
		xjust=xjust, yjust=yjust,
	legend=c('CDF estimate',paste(conflev,'% Confidence Limits', sep='')),
	lty=c(1,3), bty='n', cex=1)

if(!is.null(ylab.r)) {
  # compute right side labels to match right side tick marks
  yl.lab <- seq(par('yaxp')[1], par('yaxp')[2], len=par('yaxp')[3]+1 )
  yr.lab <- interp.axis(yl.lab, cdfdata[,2], cdfdata[,5])
  axis(side=4, at=yl.lab, labels=as.character(round(yr.lab)) )
  mtext( ylab.r, side=4, line=2, cex=par('cex'))
}

par(op)
}


interp.axis <- function(pctval, cdfest.p, cdfest.u) {
#
# Arguments
#
#  pctval    Proportions that are basis for interpolating estimated cdfest.u
#            Must be based on estimated cdf values in terms of proportions.
#  cdfest.p  estimated cdf values in terms of proportions.
#  cdfest.u  estimated cdf values in terms of units.
#
# Output
#   given a set of percentiles calculate the corresponding cdf estimated value
#   in terms of "unit" estimates.
#
# Details
#  Assumes that cdfest.p and cdfest.u are strictly increasing.  If pctval is 
#  less than first cdfest.p, then assumes 0 for both cdfest.p and cdfest.u

# find number of cdf estimates
nvec <- 1:length(cdfest.p)
rslt <- numeric(0)
for (j in 1:length(pctval) ) {
  high <- ifelse(length(nvec[cdfest.p >= pctval[j]]) > 0, 
                  min(nvec[cdfest.p >= pctval[j]]), nvec[length(nvec)] )
  low <- ifelse(length(nvec[cdfest.p <= pctval[j]]) >  0,
                  max(nvec[cdfest.p <= pctval[j]]), NA)
   
   if(is.na(low) ) {
      ulow <- 0
      plow <- 0
      pdis <- (pctval[j] - plow)/(cdfest.p[high] - plow)
    	rslt[j] <- ulow + pdis * (cdfest.u[high] - ulow)
   }
   else {
    if( high > low) {
    	pdis <- (pctval[j] - cdfest.p[low])/(cdfest.p[high] - cdfest.p[low])
    	rslt[j] <- cdfest.u[low] + pdis * (cdfest.u[high] - cdfest.u[low])
    }
    else
      rslt[j] <- cdfest.u[high]
    }
}
return(rslt)
}


interp.cdf <- function(pctval, cdfest.p, cdfest.u) {
#
# Arguments
#
#  pctval    Proportions that are basis for interpolating estimated cdfest.u
#            Must be based on estimated cdf values in terms of proportions.
#  cdfest.p  estimated cdf values in terms of proportions.
#  cdfest.u  estimated cdf values in terms of units.
#
# Output
#   given a set of percentiles calculate the corresponding cdf estimated value
#   in terms of "unit" estimates.
#
# Details
#  Assumes that cdfest.p and cdfest.u are strictly increasing

# find number of cdf estimates
nvec <- 1:length(cdfest.p)
rslt <- numeric(0)
for (j in 1:length(pctval) ) {
  high <- ifelse(length(nvec[cdfest.p >= pctval[j]]) > 0, 
                  min(nvec[cdfest.p >= pctval[j]]), NA )
  low <- ifelse(length(nvec[cdfest.p <= pctval[j]]) >  0,
                  max(nvec[cdfest.p <= pctval[j]]), NA)
   if (is.na(high)) {
      rslt[j] <- NA
   }
   else if(is.na(low) ) {
     rslt[j] <- cdfest.u[high]
   }
   else {
    if( high > low) {
    	pdis <- (pctval[j] - cdfest.p[low])/(cdfest.p[high] - cdfest.p[low])
    	rslt[j] <- cdfest.u[low] + pdis * (cdfest.u[high] - cdfest.u[low])
    }
    else
      rslt[j] <- cdfest.u[high]
    }
}
return(rslt)
}

